@@ -1,3 +1,152 @@
+#' Shrink log2 fold changes
+#'
+#' This function adds shrunken log2 fold changes (LFC) to a
+#' results table which was run without LFC moderation.
+#' Note: this function is still being prototyped.
+#'
+#' @param dds a DESeqDataSet object, which has been run through
+#' \code{\link{DESeq}}, or at the least, \code{\link{estimateDispersions}}
+#' @param coef the number of the coefficient (LFC) to shrink,
+#' consult \code{resultsNames(dds)} after running \code{DESeq(dds, betaPrior=FALSE)}.
+#' only \code{coef} or \code{contrast} can be specified, not both
+#' @param contrast see argument description in \code{\link{results}}.
+#' only \code{coef} or \code{contrast} can be specified, not both
+#' @param res a DESeqResults object (can be missing)
+#' @param type at this time, ignored argument, because only one
+#' shrinkage estimator, but more to come!
+#'
+#' @return if \code{res} is not missing, a DESeqResults object with
+#' the \code{log2FoldChange} column replaced with a shrunken LFC.
+#' If \code{res} is missing, just the shrunken LFC vector.
+#'
+#' @export
+#' 
+#' @examples
+#'
+#'  dds <- makeExampleDESeqDataSet(betaSD=1)
+#'  dds <- DESeq(dds, betaPrior=FALSE)
+#'  res <- results(dds)
+#'  res.shr <- lfcShrink(dds=dds, coef=2, res=res)
+#'  res.shr <- lfcShrink(dds=dds, contrast=c("condition","B","A"), res=res)
+#' 
+lfcShrink <- function(dds, coef, contrast, res, type="normal") {
+  if (is.null(dispersions(dds))) {
+    stop("lfcShrink requires dispersion estimates, first call estimateDispersions()")
+  }
+
+  # match the shrinkage type
+  type <- match.arg(type, choices=c("normal"))
+
+  # fit MLE coefficients... TODO skip this step
+  dds <- estimateMLEForBetaPriorVar(dds)
+
+  stopifnot(missing(coef) | missing(contrast))
+  if (missing(contrast)) {
+    modelMatrixType <- "standard"
+  } else {
+    modelMatrixType <- "expanded"
+  }
+  attr(dds,"modelMatrixType") <- modelMatrixType
+  betaPriorVar <- estimateBetaPriorVar(dds)
+
+  dds.shr <- nbinomWaldTest(dds,
+                            betaPrior=TRUE,
+                            betaPriorVar=betaPriorVar,
+                            modelMatrixType=modelMatrixType,
+                            quiet=TRUE)
+
+  if (missing(contrast)) {
+    rn <- resultsNames(dds.shr)
+    res.shr <- results(dds.shr, name=rn[coef])
+  } else {
+    res.shr <- results(dds.shr, contrast=contrast)
+  }
+  
+  if (!missing(res)) {
+    res <- res[,c("baseMean","log2FoldChange","stat","pvalue","padj")]
+    res$log2FoldChange <- res.shr$log2FoldChange
+    mcols(res)$description[2] <- mcols(res.shr)$description[2]
+    return(res)
+  } else {
+    return(res.shr$log2FoldChange)
+  }
+}
+
+#' Unmix samples using loss in a variance stabilized space
+#'
+#' Unmixes samples in \code{x} according to \code{pure} components,
+#' using numerical optimization. The components in \code{pure}
+#' are added on the scale of gene expression (either normalized counts, or TPMs).
+#' The loss function when comparing fitted expression to the
+#' samples in \code{x} occurs in a variance stabilized space.
+#' This task is sometimes referred to as "deconvolution",
+#' and can be used, for example, to identify contributions from
+#' various tissues.
+#' Note: if the \code{pbapply} package is installed a progress bar
+#' will be displayed while mixing components are fit.
+#'
+#' @param x normalized counts or TPMs of the samples to be unmixed
+#' @param pure normalized counts or TPMs of the "pure" samples
+#' @param alpha for normalized counts, the dispersion of the data
+#' when a negative binomial model is fit. this can be found by examining
+#' the asymptotic value of \code{dispersionFunction(dds)}, when using
+#' \code{fitType="parametric"} or the mean value when using
+#' \code{fitType="mean"}.
+#' @param shift for TPMs, the shift which approximately stabilizes the variance
+#' of log shifted TPMs. Can be assessed with \code{vsn::meanSdPlot}.
+#' @param loss either 1 (for L1) or 2 (for squared) loss function.
+#' Default is 1.
+#' @param quiet suppress progress bar. default is FALSE, show progress bar
+#' if pbapply is installed.
+#'
+#' @return mixture components for each sample (rows), which sum to 1.
+#'
+#' @export
+unmix <- function(x, pure, alpha, shift, loss=1, quiet=FALSE) {
+
+  if (missing(alpha)) stopifnot(!missing(shift))
+  if (missing(shift)) stopifnot(!missing(alpha))
+  stopifnot(missing(shift) | missing(alpha))
+  stopifnot(loss %in% 1:2)
+  stopifnot(nrow(x) == nrow(pure))
+  stopifnot(ncol(pure) > 1)
+  
+  if (requireNamespace("pbapply", quietly=TRUE) & !quiet) {
+    lapply <- pbapply::pblapply
+  }
+  
+  if (missing(shift)) {
+    stopifnot(alpha > 0)
+    # variance stabilizing transformation for NB w/ fixed dispersion alpha
+    vst <- function(q, alpha) ( 2 * asinh(sqrt(alpha * q)) - log(alpha) - log(4) ) / log(2)
+    distVST <- function(p, i, vst, alpha, loss) {
+      sum(abs(vst(x[,i], alpha) - vst(pure %*% p, alpha))^loss)
+    }
+    res <- lapply(seq_len(ncol(x)), function(i) {
+      optim(par=rep(1, ncol(pure)), fn=distVST, gr=NULL, i, vst, alpha, loss,
+            method="L-BFGS-B", lower=0, upper=100)$par
+    })
+  } else {
+    stopifnot(shift > 0)
+    # VST of shifted log
+    vstSL <- function(q, shift) log(q + shift)
+    distSL <- function(p, i, vst, shift, loss) {
+      sum(abs(vstSL(x[,i], shift) - vstSL(pure %*% p, shift))^loss)
+    }
+    res <- lapply(seq_len(ncol(x)), function(i) {
+      optim(par=rep(1, ncol(pure)), fn=distSL, gr=NULL, i, vstSL, shift, loss,
+            method="L-BFGS-B", lower=0, upper=100)$par
+    })
+  }
+
+  mix <- do.call(rbind, res)
+  mix <- mix / rowSums(mix)
+  colnames(mix) <- colnames(pure)
+  
+  return(mix)
+  
+}
+
 #' Collapse technical replicates in a RangedSummarizedExperiment or DESeqDataSet
 #'
 #' Collapses the columns in \code{object} by summing within levels
@@ -340,14 +489,7 @@ DESeqParallel <- function(object, test, fitType, betaPrior, full, reduced,
 
     # if so:
 
-    # need to set standard model matrix for LRT with beta prior
-    if (test == "LRT") {
-      attr(object, "modelMatrixType") <- "standard"
-      attr(objectNZ, "modelMatrixType") <- "standard"
-      modelMatrixType <- "standard"
-    }
-
-    # also if explicitly set
+    # if explicitly set
     if (!is.null(modelMatrixType) && modelMatrixType == "standard") {
       attr(object, "modelMatrixType") <- "standard"
       attr(objectNZ, "modelMatrixType") <- "standard"
@@ -371,18 +513,12 @@ DESeqParallel <- function(object, test, fitType, betaPrior, full, reduced,
 
     # the third parallel execution: the final GLM and statistics
     if (!quiet) message(paste("fitting model and testing:",nworkers,"workers"))
-    if (test == "Wald") {
-
-      objectNZ <- do.call(rbind, bplapply(levels(idx), function(l) {
-        nbinomWaldTest(objectNZ[idx == l,,drop=FALSE], betaPriorVar=betaPriorVar,
-                       quiet=TRUE, modelMatrixType=modelMatrixType)
-      }, BPPARAM=BPPARAM))
-    } else if (test == "LRT") {
-      objectNZ <- do.call(rbind, bplapply(levels(idx), function(l) {
-        nbinomLRT(objectNZ[idx == l,,drop=FALSE], full=full, reduced=reduced,
-                  betaPrior=betaPrior, betaPriorVar=betaPriorVar, quiet=TRUE)
-      }, BPPARAM=BPPARAM))
-    }
+    objectNZ <- do.call(rbind, bplapply(levels(idx), function(l) {
+      nbinomWaldTest(objectNZ[idx == l,,drop=FALSE],
+                     betaPrior=TRUE,
+                     betaPriorVar=betaPriorVar,
+                     quiet=TRUE, modelMatrixType=modelMatrixType)
+    }, BPPARAM=BPPARAM))
     
   } else {
     
@@ -396,7 +532,7 @@ DESeqParallel <- function(object, test, fitType, betaPrior, full, reduced,
                                               dispPriorVar=dispPriorVar, quiet=TRUE, modelMatrix=modelMatrix)
         # replace design
         if (noReps) design(objectNZSub) <- designIn
-        nbinomWaldTest(objectNZSub, betaPrior=betaPrior,
+        nbinomWaldTest(objectNZSub, betaPrior=FALSE,
                        quiet=TRUE, modelMatrix=modelMatrix, modelMatrixType="standard")
       }, BPPARAM=BPPARAM))
     } else if (test == "LRT") {
