@@ -369,18 +369,26 @@ setReplaceMethod("normalizationFactors", signature(object="DESeqDataSet", value=
                    object
                  })
 
-estimateSizeFactors.DESeqDataSet <- function(object, type=c("ratio","iterate"),
-                                             locfunc=stats::median, geoMeans, controlGenes, normMatrix) {
-  type <- match.arg(type, c("ratio","iterate"))
+estimateSizeFactors.DESeqDataSet <- function(object, type=c("ratio","poscounts","iterate"),
+                                             locfunc=stats::median,
+                                             geoMeans, controlGenes, normMatrix) {
+  type <- match.arg(type, c("ratio","poscounts","iterate"))
   # Temporary hack for backward compatibility with "old" DESeqDataSet
   # objects. Remove once all serialized DESeqDataSet objects around have
   # been updated.
-  if (!.hasSlot(object, "rowRanges"))
+  if (!.hasSlot(object, "rowRanges")) {
     object <- updateObject(object)
+  }
   object <- sanitizeColData(object)
   if (type == "iterate") {
     sizeFactors(object) <- estimateSizeFactorsIterate(object)
   } else {
+    if (type == "poscounts") {
+      geoMeanNZ <- function(x) {
+        if (all(x == 0)) { 0 } else { exp( sum(log(x[x > 0])) / length(x) ) }
+      }
+      geoMeans <- apply(counts(object), 1, geoMeanNZ)
+    }
     if ("avgTxLength" %in% assayNames(object)) {
       nm <- assays(object)[["avgTxLength"]]
       nm <- nm / exp(rowMeans(log(nm))) # divide out the geometric mean
@@ -434,12 +442,17 @@ estimateSizeFactors.DESeqDataSet <- function(object, type=c("ratio","iterate"),
 #' @aliases estimateSizeFactors estimateSizeFactors,DESeqDataSet-method
 #' 
 #' @param object a DESeqDataSet
-#' @param type either "ratio" or "iterate". "ratio" uses the standard
-#' median ratio method introduced in DESeq. The size factor is the
-#' median ratio of the sample over a pseudosample: for each gene, the geometric mean
-#' of all samples. "iterate" offers an alternative estimator, which can be
-#' used even when all genes contain a sample with a zero. This estimator
-#' iterates between estimating the dispersion with a design of ~1, and
+#' @param type Method for estimation: either "ratio", "poscounts", or "iterate".
+#' "ratio" uses the standard median ratio method introduced in DESeq. The size factor is the
+#' median ratio of the sample over a "pseudosample": for each gene, the geometric mean
+#' of all samples.
+#' "poscounts" and "iterate" offer alternative estimators, which can be
+#' used even when all genes contain a sample with a zero (a problem for the
+#' default method, as the geometric mean becomes zero, and the ratio undefined).
+#' The "poscounts" estimator deals with a gene with some zeros, by calculating a
+#' modified geometric mean by taking the n-th root of the product of the non-zero counts.
+#' This evolved out of use cases with Paul McMurdie's phyloseq package for metagenomic samples.
+#' The "iterate" estimator iterates between estimating the dispersion with a design of ~1, and
 #' finding a size factor vector by numerically optimizing the likelihood
 #' of the ~1 model.
 #' @param locfunc a function to compute a location for a sample. By default, the
@@ -740,10 +753,11 @@ coef.DESeqDataSet  <- function(object, SE=FALSE, ...) {
 #' \method{summary}{DESeqResults}(object, alpha, \dots)
 #' 
 #' @param object a \code{\link{DESeqResults}} object
-#' @param alpha the adjusted p-value cutoff. if not set, this
+#' @param alpha the adjusted p-value cutoff. If not set, this
 #' defaults to the \code{alpha} argument which was used in
 #' \code{\link{results}} to set the target FDR for independent
-#' filtering.
+#' filtering, or if independent filtering was not performed,
+#' to 0.1.
 #' @param ... additional arguments
 #'
 #' @docType methods
